/*
 *  Copyright (c) 1999 by Niels Gorisse <niels@bonneville.nl>
 *  Modifications for WireProvider and Large Sysex (c) 2000
 *  by Gerrit Gehnen <Gerrit.Gehnen@gmx.de>
 *  Modifications for Larger Sysex (c) 2001
 *  by Brian Klock <jsynthlib@overwhelmed.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package WireProvider;


import javax.sound.midi.*;
import java.util.Vector;
import java.io.OutputStream;
import core.*;
//import WireNative;
/**
 * WireInput represents a MIDI input device.
 * Active sensing is filtered out (you will not see it back). When
 * an exception is thrown, the device is stopped automatically.
 * @author    <A HREF="mailto:niels@bonneville.nl">Niels Gorisse</A>, 22 april 1999, Bonneville
 * @version   WireInput 0.97
 **/


public final class WireInput extends Thread implements MidiDevice
{
private WireNative myinput;
private String muststop = "Not Yet";
private WireDeviceInfo winfo=new WireDeviceInfo("WireInput","Gerrit Gehnen","WireInput","0.1");
// Following methods are from interface MidiDevice

private Vector transmitters;

class WireInputTransmitter implements Transmitter
  {
  private Receiver r;
  
  WireInputTransmitter()
    {
//        System.out.println("WireInputTransmitter Konstruktor");
    r=null;
  }
  
    /** Getter for property receiver.
     * @return Value of property receiver.
     */
  
    public Receiver getReceiver ()
    {
    return r;
    }
    /** Setter for property receiver.
     * @param receiver New value of property receiver.
     */
    public void setReceiver (Receiver receiver)
    {
//      System.out.println("WireInputTransmitter: setReceiver");
    r=receiver;
    }

    /** Closes the Port of the Receiver
    *
    **/
    public void close()
    {
//    System.out.println("WireInputTransmitter: close");
    r=null;
    transmitters.removeElement(this);
    if (transmitters.isEmpty())
       WireInput.this.close();
        
    }
public void finalize ()
    {
//        System.out.println("WireInputTransmitter: finalize");
        close();
    }
}


  /** Creates a new WireInput object and connect it to the JNI
   * @throws Exception if there are problems
   */
  public WireInput () throws Exception
  {
//    System.out.println ("WireInput: Konstruktor");
    myinput = new WireNative (0);
//    System.out.println ("Vor start");
    transmitters= new Vector ();
    start ();
  }

/**
 * Close this device. Closing never throws an exception, because when any exception occurs,
 * the device is closed anyway to prevent that the device is located even after the program
 * has quit.
 **/

public void close()
	{
  //                      System.out.println("WireInput: close");
	for (int i=0;i<transmitters.size ();i++)
          {
          ((WireInputTransmitter)transmitters.get (i)).close ();
         }
        transmitters.removeAllElements ();
//        System.out.println("WireInput: vor muststopabfrage "+muststop);
        synchronized(muststop){/* if (muststop.equals("running"))*/ muststop="muststop"; /*else return;*/} // This was necessary, because under W2K lockups occure due to race conditions
	try{sleep(11);} catch (InterruptedException e) {;} // prevents collision of threads
//	System.out.println("WireInput: vor myinput.close");
        try{myinput.close();}catch(Exception e2) {;}
  //       System.out.println("WireInput: close End");
	}



        /** Obtains information about the device, including its Java class and Strings containing its name, vendor, and description.
         * @return Device Info
         */
        public MidiDevice.Info getDeviceInfo ()
        {
          return winfo;

        }


        /** Returns weither this device is open or not.
         * @return true, if device is open
         * false, if device is not open
         */

public boolean isOpen()
	{
        synchronized(muststop){return muststop.equals("running");}}


        /** Open this device.
         * @throws MidiUnavailableException if there is no Device availiable
         */

public void open() throws MidiUnavailableException
	{
//        System.out.println("WireInput open muststop: "+muststop);
	synchronized(muststop){if (muststop.equals("running")) return;}
	if (myinput.getport()==-1) return;
	myinput.open();
	synchronized(muststop){muststop="running";}
	synchronized(this){notify();}
	}
/** Required for Threads - This is the primary function for receiving data from the 
*   JNI.
*
**/
public synchronized void run()
{
byte[] inhere = new byte[65535];
while(true)
	{
//            System.out.println("WireInput Run- Schleife");
	try {
		this.setPriority(2);
		synchronized(muststop){muststop="not yet";}
		this.wait();
		}
	catch (InterruptedException e) {;}
	try	{
		this.setPriority(6);
		while (true)
			{
  //                      System.out.print("I");
			int size= myinput.readLine(inhere);
                        
			if (size!=0)
                            {
  //                          System.out.println("read "+size);
                            write(inhere,size);
                            }
			sleep(2); // necessary? // TODO
			synchronized(muststop)
                            {
                            if (muststop.equals("muststop"))
                                {
//                                System.out.println("Thread: Muststop!");
                                break;
                                }
                            }
			}
		}
	catch (InterruptedException e) {;}
	catch (Exception e) {System.out.println("WireInput: error while getting MIDI input messages: "+e.getMessage());}
  //      System.out.println("WireInput Thread ausgelaufen");
	}}


        /** Return how many Transmitters are present in the system.
         * @return The numbers of Transmitters
         * -1, if the number is unlimited
         */

public int getMaxTransmitters()
	{
        return -1;
        }

        /** Set which port this device represents.
         * 
         * @param port The physical portnumber
         * @throws Exception if there are problems with the port
         */

public void setport(int port) throws Exception 
  {
  myinput.setport(port);
  winfo=new WireDeviceInfo("Wire: "+myinput.devicename(port),"WireProvider","Wire Input","0.1");
  }

  /** Wire also keeps also track of the current 'port' (-1 is none).
   * @return Number of the actual internal port
   */

public int getport() {return myinput.getport();}

// NU DE TRANSMITTER:

private void write(byte[] cbuf,int len)
	{ // Jchar = UNSIGNED!! (0/veel) byte = unsigned (ofwel: 0/256!!
//          System.out.println("WireInput: write");
//	if (receiver==null) return;
//	synchronized(receiver)
		{
		int lengte;
		char buf1,buf2,buf3,buf4;
		for (int x=1;x<len;)
			{
			lengte =(((int)cbuf[x+2])&0xff)+((int)cbuf[x+3]*0x100);  //Length is now 16 Bit
/*
                        System.out.print("WireInput "+Integer.toHexString(cbuf[x])+" "+
                                        Integer.toHexString(cbuf[x+1])+" "+
                                        Integer.toHexString(cbuf[x+2])+" "+
                                        Integer.toHexString(cbuf[x+3])+" "+
                                        Integer.toHexString(cbuf[x+4])+" "+
                                        Integer.toHexString(cbuf[x+5])+" "+
                                         Integer.toHexString(cbuf[x+6])+" "+
                                         Integer.toHexString(cbuf[x+7])+" "+
                                         Integer.toHexString(cbuf[x+8])+" "+
                                         Integer.toHexString(cbuf[x+9])+" "+
                                         Integer.toHexString(cbuf[x+10])+" "+
                                         Integer.toHexString(cbuf[x+11])+" ");
                        System.out.println("Lengte: "+lengte);
*/
                        if (lengte==0)
                          System.exit(0);
                        long time = ((long)cbuf[x+4]&0xff)+
                                                      (((long)cbuf[x+5]&0xff)<<8)+
                                                      (((long)cbuf[x+6]&0xff)<<16)+
                                                      (((long)cbuf[x+7]&0xff)<<24);
//System.out.println("Java-Timestamp: "+Long.toHexString(time));
			switch((byte)(cbuf[x] ))
				{
				case 63:
					{
					ShortMessage tosend=new ShortMessage();
                                          
					try	{
//                                            System.out.println("Short message: "+(int)cbuf[x+4]);
						tosend.setMessage(((int)cbuf[x+8])&0xff,((int)cbuf[x+9]),((int)cbuf[x+10]));
						}
					catch(InvalidMidiDataException e) {System.out.println("Oops, ShortEvent problem: "+e.getMessage());break;}

                                        try 
                                        {
                                        for(int i1 = 0; i1 < transmitters.size(); i1++)
                                          {
                                          Receiver receiver = ((Transmitter)transmitters.elementAt(i1)).getReceiver();
                                          if(receiver != null)
                                            receiver.send(tosend, time*1000);
                                          }

                                      }
                                          catch (Exception e) {System.out.println("Send: Exception"+e.getMessage());}
                                        break;
                                        }
				case 64:
					{
					SysexMessage tosend=new SysexMessage();
					byte arree[] = new byte[lengte-8]; // We don't need the header
					//System.out.print("Java: Got sysex: ");
                                        for (int r=0;r<lengte-8;r++) 
                                          {
                                          Integer i=new Integer(0);
                                          arree[r]=(byte)cbuf[x+8+r]; 
                                          //System.out.print(i.toHexString(arree[r])+" ");
                                          }
//                                         System.out.println("");
//					long time = 0;
					try
						{
                                                tosend.setMessage(arree,lengte-8);
						}
                                        catch(InvalidMidiDataException e)

//// Beginning of Code section modified by Brian Klock April 2001
//// To handle larger sysex messages
                                          {
                                            byte arree2[] = new byte[lengte-7];
                                            System.arraycopy(arree,0,arree2,1,arree.length);
                                            arree2[0]=((byte)(0xF7));
                                            try {
                                              tosend.setMessage(arree2,lengte-7);
                                            }catch(Exception e1) {System.out.println("Oops!, SysexMessage problem: "+e.getMessage());}
                                          }
//// End of modified code section
                                        try {
                                        for(int i1 = 0; i1 < transmitters.size(); i1++)
                                          {
                                          Receiver receiver = ((Transmitter)transmitters.elementAt(i1)).getReceiver();
                                          if(receiver != null)
                                            receiver.send(tosend, time*1000);
                                          }
                                        }
                                        catch (Exception e) {System.out.println("Send: Exception"+e.getMessage());}
					break;
					}
				}
			x+=lengte;
			}
		}
	}

// unimplemented:
        /** currently unimplemented
         * @return always -1
         */
        public long getMicrosecondPosition ()
        {
        long temp;
        temp=myinput.getPosition();
        return temp;
        }

        /** Returns a Transmitter
         * @throws MidiUnavailableException if there is a problem 
         * @return the Transmitter
         */
        public Transmitter getTransmitter () throws MidiUnavailableException
        {
    //    System.out.println("WireInput GetTransmitter");
         if (!isOpen())
          open();
/*        else
          System.out.println("WireInput: is already open");
*/
        WireInputTransmitter wor=new WireInputTransmitter();
        transmitters.addElement(wor);
        return wor;
        
        }
        
        
// not necessary:
        /** Required for Interface MidiDevice, but there is no Receiver
         * @return always 0, because this object represents no Receiver
         */
        public int getMaxReceivers ()
        {return 0;}

/** Needed for MidiDevice, but throws always an exception, because there is no Receiver
 * @throws MidiUnavailableException always
 * @return nothing
 */
public Receiver getReceiver () throws MidiUnavailableException
{
  throw new MidiUnavailableException ("Sorry, WireInput has no Receivers");
}

public void finalize()
  {
//    System.out.println("WireInput finalize");
  close();
}
} // end of WireInput


